<?php
    namespace Teknisa\CADPED\Controllers;
    
    use Teknisa\CADPED\Controllers\TeknisaCrud;
    use Teknisa\Libs\Controller\CRUD;
    use Zeedhi\Framework\DTO;
    use Zeedhi\Framework\DTO\Response\Error;
    use Zeedhi\Framework\DataSource\Manager;
    use Zeedhi\Framework\DataSource\DataSet;
    use Zeedhi\Framework\DataSource\ParameterBag;
    use Zeedhi\Framework\DataSource\FilterCriteria;
    use Teknisa\CADPED\Services\PED00900RamoService;

    class PED00900RamoAtividade extends TeknisaCrud {
        
        protected $dataSourceName = 'ramo_atividade';
        protected $parameterBagColumns = array();
        protected $environment;
        protected $crudService;
        protected $connection;
        
        public function __construct($crudService, ParameterBag $parameterBag, $environment, $connection, PED00900RamoService $service){
            $this->crudService  = $crudService;
            $this->service      = $service;
            $this->environment  = $environment;
            $this->parameterBag = $parameterBag;
            $this->connection   = $connection;
            parent::__construct($crudService, $parameterBag);
        
        }
        
        //DataSource ramo_atividade
        public function findRamoAtividade(DTO\Request $request, DTO\Response $response){
            $this->dataSourceName = 'ramo_atividade';
            parent::find($request,$response);
        }
        
        //busca no dataSource paramativcli o parametro na paramgeral correspondente à mascara
        public function findParamAtivCli(DTO\Request $request, DTO\Response $response){
            $this->dataSourceName = 'paramativcli';
            parent::findParams($request,$response);
        }
        
        public function saveRamo(DTO\Request $request, DTO\Response $response){
            $this->dataSourceName = 'ativcliente';
    		$rowToSave = $request->getDataSet()->getRows()[0];
    		
            if(!empty($rowToSave)){
    			try{
    	            //Se for um novo registro, fazer as validações necessárias para inserir
    	            if($rowToSave['__is_new']){ 
    	            
    	                //Se(retornar qualquer coisa diferente de false){ ('Código já cadastrado') }
    	                if(!$this->verificaSeExiste($rowToSave['CDARVATIVCLI'])){
    	                    
        	                //Verifica se o nivel anterior foi cadastrado ou se é o primeiro nivel
        	                if($this->verificaRamoNivelAnterior($rowToSave['CDARVATIVCLI']) || $rowToSave['CDNVATIVCLI'] === '1'){   
        	                    
        	                    $rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
    		                    $this->crudService->save($this->dataSourceName, $rows);
    		                    
            	                $sucesso = array(
        	                        'message' => 'Cadastrado com sucesso!'
        	                    );
        	                    $response->addDataSet(new DataSet ('success', $sucesso));
        	                }else{
        	                    //NIVEL ANTERIOR NÃO CADASTRADO
        	                    $erro = array(
        	                        'message' => 'Nível anterior não cadastrado!'
        	                    );
        	                    $response->addDataSet(new DataSet ('error', $erro));
        	                }
    	                }else{
    	                    // JA EXISTE
    	                    $erro = array(
    	                        'message' => 'Código já cadastrado!'
    	                    );
    	                    $response->addDataSet(new DataSet ('error', $erro));
    	                }
    	            }else{ // se não for novo, atualiza
    	                
    	                $rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
    	                
	                    $this->crudService->save($this->dataSourceName, $rows);
    	                $sucesso = array(
	                        'message' => 'Alterado com sucesso!'
	                    );
	                    $response->addDataSet(new DataSet ('success', $sucesso));
    	            }
	            }catch(Exception $e){
                    throw $e;
                }
            }
        }
        
        private function verificaRamoNivelAnterior($cdarvativcli){
            $parametro = array(
                'CDARVATIVCLI' => $cdarvativcli
            );
            return $this->service->verificaRamoNivelAnterior($parametro);
        }
        
        private function verificaSeExiste($cdarvativcli){
            $parametro = array(
                'CDARVATIVCLI' => $cdarvativcli
            );
            return $this->service->verificaSeExiste($parametro);
        }
        
        private function verificaRamoProximoNivel($cdarvativcli){
            $parametro = array(
                'CDARVATIVCLI' => $cdarvativcli
            );
            return $this->service->verificaProximoNivel($parametro);
        }
        
        public function deleteRamo(DTO\Request $request, DTO\Response $response){
    		$dataSet = $request->getDataSet();
    		$rowToDelete = $dataSet->getRows()[0];
    		
    		if(!$this->verificaRamoProximoNivel($rowToDelete['CDARVATIVCLI'])){
        		$parametros =  array(
                    'NRORG'        =>  $rowToDelete['NRORG'],
                    'CDARVATIVCLI' =>  $rowToDelete['CDARVATIVCLI']
                );
                try{
                    $dataSet->setDataSourceName('ativcliente');
                    $this->crudService->delete($response, $dataSet);
                }catch(Exception $e){
                    throw $e;
                }
    		}else{
                $erro = array(
                    'message' => 'Exclusão bloquada! Este item possui subitens!'
                );
                $response->addDataSet(new DataSet ('error', $erro));
    		}
        }
        
        public function geraCodigoRamo(DTO\Request $request, DTO\Response $response){
            $row        = $request->getRow();       
            $isMaxLevel = $row['ISMAXNIVEL'];
            $codigo     = $row['CDATIVCLI'];
            
            if($isMaxLevel){
                $resultado = $this->verificaRamoNivelAnterior($codigo);
                $codigo = $resultado['CDARVATIVCLI'];
            }
            $result = $this->verificaRamoProximoNivel($codigo);
            if($result){
                $maxCdRamo = max($result)['CDATIVCLI'];
                $response->addDataset(new DataSet('CODIGO', array($maxCdRamo+1)));
            }else{
                $response->addDataset(new DataSet('CODIGO', array(false)));
            }
    		
        }
    }    
?>